@@ -38,7 +38,6 @@ from .._utils import (
     StreamType,
     b_,
     deprecate_no_replacement,
-    deprecation_with_replacement,
     logger_warning,
     read_non_whitespace,
     read_until_regex,
@@ -74,22 +73,21 @@ class PdfObject(PdfObjectProtocol):
         ignore_fields: Optional[Sequence[Union[str, int]]] = (),
     ) -> "PdfObject":
         """
-        clone object into pdf_dest (PdfWriterProtocol which is an interface for PdfWriter)
-        force_duplicate: in standard if the object has been already cloned and reference,
-            the copy is returned; when force_duplicate == True,
-            a new copy is always performed
-        ignore_fields : list/tuple of Fields names (for dictionaries that will
-            be ignored during cloning (apply also to childs duplication)
-            if fields are to be considered for a limited number of levels
-            you have to add it as integer:
-            eg  [1,"/B","/TOTO"] means "/B" will be ignored at first level only
-            but "/TOTO" on all levels
-        in standard, clone function call _reference_clone (see _reference)
+        Clone object into pdf_dest (PdfWriterProtocol which is an interface for PdfWriter).
+
+        By default, this method will call ``_reference_clone`` (see ``_reference``).
+
 
         Args:
-          pdf_dest:
-          force_duplicate:  (Default value = False)
-          ignore_fields:
+          pdf_dest: Target to clone to.
+          force_duplicate: By default, if the object has already been cloned and referenced,
+            the copy will be returned; when ``True``, a new copy will be created.
+            (Default value = ``False``)
+          ignore_fields: List/tuple of field names (for dictionaries) that will be ignored
+            during cloning (applies to children duplication as well). If fields are to be
+            considered for a limited number of levels, you have to add it as integer, for
+            example ``[1,"/B","/TOTO"]`` means that ``"/B"`` will be ignored at the first
+            level only but ``"/TOTO"`` on all levels.
 
         Returns:
           The cloned PdfObject
@@ -147,10 +145,6 @@ class PdfObject(PdfObjectProtocol):
         """Resolve indirect references."""
         return self
 
-    def getObject(self) -> Optional["PdfObject"]:  # deprecated
-        deprecation_with_replacement("getObject", "get_object", "3.0.0")
-        return self.get_object()
-
     def write_to_stream(
         self, stream: StreamType, encryption_key: Union[None, str, bytes] = None
     ) -> None:
@@ -185,20 +179,9 @@ class NullObject(PdfObject):
             raise PdfReadError("Could not read Null object")
         return NullObject()
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
     def __repr__(self) -> str:
         return "NullObject"
 
-    @staticmethod
-    def readFromStream(stream: StreamType) -> "NullObject":  # deprecated
-        deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
-        return NullObject.read_from_stream(stream)
-
 
 class BooleanObject(PdfObject):
     def __init__(self, value: Any) -> None:
@@ -239,12 +222,6 @@ class BooleanObject(PdfObject):
         else:
             stream.write(b"false")
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
     @staticmethod
     def read_from_stream(stream: StreamType) -> "BooleanObject":
         word = stream.read(4)
@@ -256,11 +233,6 @@ class BooleanObject(PdfObject):
         else:
             raise PdfReadError("Could not read Boolean object")
 
-    @staticmethod
-    def readFromStream(stream: StreamType) -> "BooleanObject":  # deprecated
-        deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
-        return BooleanObject.read_from_stream(stream)
-
 
 class IndirectObject(PdfObject):
     def __init__(self, idnum: int, generation: int, pdf: Any) -> None:  # PdfReader
@@ -309,10 +281,36 @@ class IndirectObject(PdfObject):
         return self
 
     def get_object(self) -> Optional["PdfObject"]:
-        obj = self.pdf.get_object(self)
-        if obj is None:
-            return None
-        return obj.get_object()
+        return self.pdf.get_object(self)
+
+    def __deepcopy__(self, memo: Any) -> "IndirectObject":
+        return IndirectObject(self.idnum, self.generation, self.pdf)
+
+    def _get_object_with_check(self) -> Optional["PdfObject"]:
+        o = self.get_object()
+        # the check is done here to not slow down get_object()
+        if isinstance(o, IndirectObject):
+            raise PdfStreamError(
+                f"{self.__repr__()} references an IndirectObject {o.__repr__()}"
+            )
+        return o
+
+    def __getattr__(self, name: str) -> Any:
+        # Attribute not found in object: look in pointed object
+        try:
+            return getattr(self._get_object_with_check(), name)
+        except AttributeError:
+            raise AttributeError(
+                f"No attribute {name} found in IndirectObject or pointed object"
+            )
+
+    def __getitem__(self, key: Any) -> Any:
+        # items should be extracted from pointed Object
+        return self._get_object_with_check()[key]  # type: ignore
+
+    def __str__(self) -> str:
+        # in this case we are looking for the pointed data
+        return self.get_object().__str__()
 
     def __repr__(self) -> str:
         return f"IndirectObject({self.idnum!r}, {self.generation!r}, {id(self.pdf)})"
@@ -338,12 +336,6 @@ class IndirectObject(PdfObject):
             )
         stream.write(f"{self.idnum} {self.generation} R".encode())
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
     @staticmethod
     def read_from_stream(stream: StreamType, pdf: Any) -> "IndirectObject":  # PdfReader
         idnum = b""
@@ -371,13 +363,6 @@ class IndirectObject(PdfObject):
             )
         return IndirectObject(int(idnum), int(generation), pdf)
 
-    @staticmethod
-    def readFromStream(
-        stream: StreamType, pdf: Any  # PdfReader
-    ) -> "IndirectObject":  # deprecated
-        deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
-        return IndirectObject.read_from_stream(stream, pdf)
-
 
 FLOAT_WRITE_PRECISION = 8  # shall be min 5 digits max, allow user adj
 
@@ -431,12 +416,6 @@ class FloatObject(float, PdfObject):
             )
         stream.write(self.myrepr().encode("utf8"))
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
 
 class NumberObject(int, PdfObject):
     NumberPattern = re.compile(b"[^+-.0-9]")
@@ -472,12 +451,6 @@ class NumberObject(int, PdfObject):
             )
         stream.write(repr(self).encode("utf8"))
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
     @staticmethod
     def read_from_stream(stream: StreamType) -> Union["NumberObject", "FloatObject"]:
         num = read_until_regex(stream, NumberObject.NumberPattern)
@@ -485,13 +458,6 @@ class NumberObject(int, PdfObject):
             return FloatObject(num)
         return NumberObject(num)
 
-    @staticmethod
-    def readFromStream(
-        stream: StreamType,
-    ) -> Union["NumberObject", "FloatObject"]:  # deprecated
-        deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
-        return NumberObject.read_from_stream(stream)
-
 
 class ByteStringObject(bytes, PdfObject):
     """
@@ -532,12 +498,6 @@ class ByteStringObject(bytes, PdfObject):
         stream.write(binascii.hexlify(self))
         stream.write(b">")
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
 
 class TextStringObject(str, PdfObject):  # noqa: SLOT000
     """
@@ -618,12 +578,6 @@ class TextStringObject(str, PdfObject):  # noqa: SLOT000
                 stream.write(b_(chr(c)))
         stream.write(b")")
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
 
 class NameObject(str, PdfObject):  # noqa: SLOT000
     delimiter_pattern = re.compile(rb"\s+|[\(\)<>\[\]{}/%]")
@@ -658,16 +612,10 @@ class NameObject(str, PdfObject):  # noqa: SLOT000
             )
         stream.write(self.renumber())
 
-    def writeToStream(
-        self, stream: StreamType, encryption_key: Union[None, str, bytes]
-    ) -> None:  # deprecated
-        deprecation_with_replacement("writeToStream", "write_to_stream", "3.0.0")
-        self.write_to_stream(stream)
-
     def renumber(self) -> bytes:
         out = self[0].encode("utf-8")
         if out != b"/":
-            logger_warning(f"Incorrect first char in NameObject:({self})", __name__)
+            deprecate_no_replacement(f"Incorrect first char in NameObject, should start with '/': ({self})", "6.0.0")
         for c in self[1:]:
             if c > "~":
                 for x in c.encode("utf-8"):
@@ -718,16 +666,9 @@ class NameObject(str, PdfObject):  # noqa: SLOT000
                     f"Illegal character in Name Object ({name!r})"
                 ) from e
 
-    @staticmethod
-    def readFromStream(
-        stream: StreamType, pdf: Any  # PdfReader
-    ) -> "NameObject":  # deprecated
-        deprecation_with_replacement("readFromStream", "read_from_stream", "3.0.0")
-        return NameObject.read_from_stream(stream, pdf)
-
 
 def encode_pdfdocencoding(unicode_string: str) -> bytes:
-    retval = b""
+    retval = bytearray()
     for c in unicode_string:
         try:
             retval += b_(chr(_pdfdoc_encoding_rev[c]))
@@ -735,4 +676,4 @@ def encode_pdfdocencoding(unicode_string: str) -> bytes:
             raise UnicodeEncodeError(
                 "pdfdocencoding", c, -1, -1, "does not exist in translation table"
             )
-    return retval
+    return bytes(retval)
